/* CALCPrefsBox.cpp
 * Copyright (C) 2002 Dustin Graves <dgraves@computer.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "fox-1.6/fx.h"
#include "fox-1.6/FXElement.h"
#include "fox-1.6/FXArray.h"
#include "CALCdefs.h"
#include "CALCHelp.h"
#include "CALCWindow.h"
#include "CALCPrefsBox.h"

FXDEFMAP(CALCPrefsBox) CALCPrefsBoxMap[]={
  FXMAPFUNC(SEL_COMMAND,CALCPrefsBox::ID_DEFAULTS,CALCPrefsBox::onCmdDefaults),
  FXMAPFUNC(SEL_UPDATE,CALCPrefsBox::ID_EASTEREGGS,CALCPrefsBox::onUpdEasterEggs)
};

FXIMPLEMENT(CALCPrefsBox,FXDialogBox,CALCPrefsBoxMap,ARRAYNUMBER(CALCPrefsBoxMap))

CALCPrefsBox::CALCPrefsBox(CALCWindow* owner)
: FXDialogBox(owner,"Calculator Preferences",DECOR_TITLE|DECOR_BORDER|DECOR_CLOSE|DECOR_MENU|DECOR_RESIZE,0,0,400,0, 0,0,0,0)
{
  FXVerticalFrame* contents=new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y);
  FXHorizontalFrame* buttons=new FXHorizontalFrame(contents,PACK_UNIFORM_WIDTH|LAYOUT_BOTTOM|LAYOUT_FILL_X);
  new FXButton(buttons,"&Close",NULL,this,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 20,20);
  new FXButton(buttons,"&Defaults",NULL,this,ID_DEFAULTS,LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK);
  new FXHorizontalSeparator(contents,SEPARATOR_RIDGE|LAYOUT_BOTTOM|LAYOUT_FILL_X);

  tabbook=new FXTabBook(contents,NULL,0,LAYOUT_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  new FXTabItem(tabbook,"&Settings",NULL);
  FXVerticalFrame* settingsframe=new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  new FXLabel(settingsframe,PROG_PACKAGE" settings");
  new FXHorizontalSeparator(settingsframe,SEPARATOR_LINE|LAYOUT_FILL_X);

  FXMatrix *matrix1=new FXMatrix(settingsframe,6,MATRIX_BY_ROWS|PACK_UNIFORM_HEIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  new FXLabel(matrix1,"Display Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix1,"Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix1,"Mode:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix1,"Width:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix1,"Height:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix1,"Precision:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);

  new FXButton(matrix1,"Set...",NULL,owner,CALCWindow::ID_DISPLAYFONT,FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_Y|LAYOUT_FILL_ROW,0,0,0,0, 10,10);
  new FXButton(matrix1,"Set...",NULL,owner,CALCWindow::ID_BUTTONFONT,FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_Y|LAYOUT_FILL_ROW,0,0,0,0, 10,10);
  FXHorizontalFrame* modeframe=new FXHorizontalFrame(matrix1,FRAME_SUNKEN|LAYOUT_CENTER_Y|LAYOUT_FILL_ROW,0,0,0,0, 0,0,0,0, 0,0);
  new FXButton(modeframe,"Standard",NULL,owner,CALCWindow::ID_STANDARD,FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  new FXButton(modeframe,"Scientific",NULL,owner,CALCWindow::ID_SCIENTIFIC,FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  new FXTextField(matrix1,7,owner,CALCWindow::ID_CALCWIDTH,TEXTFIELD_INTEGER|FRAME_THICK|FRAME_SUNKEN|LAYOUT_CENTER_Y|LAYOUT_FILL_ROW);
  new FXTextField(matrix1,7,owner,CALCWindow::ID_CALCHEIGHT,TEXTFIELD_INTEGER|FRAME_THICK|FRAME_SUNKEN|LAYOUT_CENTER_Y|LAYOUT_FILL_ROW);
  FXSpinner* spinner=new FXSpinner(matrix1,5,owner,CALCWindow::ID_NUMDIGITS,JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK|LAYOUT_CENTER_Y|LAYOUT_FILL_ROW);
  spinner->setTipText(DIGITNUMDIGITS_HELP);
  spinner->setRange(0,CALCDBL_DIG+2);  //Give it a little extra

  new FXButton(matrix1,"Default",NULL,owner,CALCWindow::ID_DEFAULTDISPLAYFONT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,0,0, 10,10);
  new FXButton(matrix1,"Default",NULL,owner,CALCWindow::ID_DEFAULTBUTTONFONT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,0,0, 10,10);
  new FXButton(matrix1,"Default",NULL,owner,CALCWindow::ID_DEFAULTMODE,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,0,0, 10,10);
  new FXButton(matrix1,"Default",NULL,owner,CALCWindow::ID_DEFAULTWIDTH,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,0,0, 10,10);
  new FXButton(matrix1,"Default",NULL,owner,CALCWindow::ID_DEFAULTHEIGHT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,0,0, 10,10);
  new FXButton(matrix1,"Default",NULL,owner,CALCWindow::ID_DEFAULTNUMDIGITS,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,0,0, 10,10);


  new FXTabItem(tabbook,"C&olors",NULL);
  FXVerticalFrame* colorframe=new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  new FXLabel(colorframe,PROG_PACKAGE" colors");
  new FXHorizontalSeparator(colorframe,SEPARATOR_LINE|LAYOUT_FILL_X);

  FXHorizontalFrame *frame1=new FXHorizontalFrame(colorframe,PACK_UNIFORM_WIDTH|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0, 0,0);
  FXMatrix *matrix21=new FXMatrix(frame1,7,MATRIX_BY_ROWS|PACK_UNIFORM_HEIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  new FXLabel(matrix21,"Display:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix21,"Numeric Base:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix21,"Angle Mode:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix21,"Inverse:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix21,"Hyperbolic:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix21,"Digits:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix21,"Hex Digits:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXColorWell(matrix21,FXRGB(0,0,0),owner,CALCWindow::ID_COLOR_DISPLAY,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix21,FXRGB(0,0,0),owner,CALCWindow::ID_COLOR_BASE,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix21,FXRGB(0,0,0),owner,CALCWindow::ID_COLOR_REPRESENTATION,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix21,FXRGB(0,0,0),owner,CALCWindow::ID_COLOR_INV,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix21,FXRGB(0,0,0),owner,CALCWindow::ID_COLOR_HYP,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix21,FXRGB(0,0,0),owner,CALCWindow::ID_COLOR_DIGITS,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix21,FXRGB(0,0,0),owner,CALCWindow::ID_COLOR_HEXDIGITS,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);

  FXMatrix *matrix22=new FXMatrix(frame1,7,MATRIX_BY_ROWS|PACK_UNIFORM_HEIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  new FXLabel(matrix22,"Operators:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix22,"Functions:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix22,"Memory:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix22,"Statistics:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix22,"Erase:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix22,"Clear Entry:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix22,"Clear All:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXColorWell(matrix22,FXRGB(0,0,0),owner,CALCWindow::ID_COLOR_OPERATORS,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix22,FXRGB(0,0,0),owner,CALCWindow::ID_COLOR_FUNCTIONS,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix22,FXRGB(0,0,0),owner,CALCWindow::ID_COLOR_MEMORY,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix22,FXRGB(0,0,0),owner,CALCWindow::ID_COLOR_STATISTICS,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix22,FXRGB(0,0,0),owner,CALCWindow::ID_COLOR_BACKSPACE,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix22,FXRGB(0,0,0),owner,CALCWindow::ID_COLOR_CLEARENTRY,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix22,FXRGB(0,0,0),owner,CALCWindow::ID_COLOR_CLEARALL,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);

  new FXTabItem(tabbook,"&Text Colors",NULL);
  FXVerticalFrame* textcolorframe=new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  new FXLabel(textcolorframe,PROG_PACKAGE" text colors");
  new FXHorizontalSeparator(textcolorframe,SEPARATOR_LINE|LAYOUT_FILL_X);

  FXHorizontalFrame *frame2=new FXHorizontalFrame(textcolorframe,PACK_UNIFORM_WIDTH|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0, 0,0);
  FXMatrix *matrix31=new FXMatrix(frame2,7,MATRIX_BY_ROWS|PACK_UNIFORM_HEIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  new FXLabel(matrix31,"Display Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix31,"Numeric Base Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix31,"Angle Mode Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix31,"Invert Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix31,"Hyperbolic Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix31,"Digits Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix31,"Hex Digits Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXColorWell(matrix31,FXRGB(0,0,0),owner,CALCWindow::ID_TEXTCOLOR_DISPLAY,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix31,FXRGB(0,0,0),owner,CALCWindow::ID_TEXTCOLOR_BASE,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix31,FXRGB(0,0,0),owner,CALCWindow::ID_TEXTCOLOR_REPRESENTATION,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix31,FXRGB(0,0,0),owner,CALCWindow::ID_TEXTCOLOR_INV,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix31,FXRGB(0,0,0),owner,CALCWindow::ID_TEXTCOLOR_HYP,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix31,FXRGB(0,0,0),owner,CALCWindow::ID_TEXTCOLOR_DIGITS,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix31,FXRGB(0,0,0),owner,CALCWindow::ID_TEXTCOLOR_HEXDIGITS,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);

  FXMatrix *matrix32=new FXMatrix(frame2,7,MATRIX_BY_ROWS|PACK_UNIFORM_HEIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  new FXLabel(matrix32,"Operators Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix32,"Functions Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix32,"Memory Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix32,"Statistics Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix32,"Erase Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix32,"Clear Entry Font:",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXLabel(matrix32,"Clear All: Font",NULL,JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X|LAYOUT_FILL_ROW);
  new FXColorWell(matrix32,FXRGB(0,0,0),owner,CALCWindow::ID_TEXTCOLOR_OPERATORS,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix32,FXRGB(0,0,0),owner,CALCWindow::ID_TEXTCOLOR_FUNCTIONS,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix32,FXRGB(0,0,0),owner,CALCWindow::ID_TEXTCOLOR_MEMORY,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix32,FXRGB(0,0,0),owner,CALCWindow::ID_TEXTCOLOR_STATISTICS,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix32,FXRGB(0,0,0),owner,CALCWindow::ID_TEXTCOLOR_BACKSPACE,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix32,FXRGB(0,0,0),owner,CALCWindow::ID_TEXTCOLOR_CLEARENTRY,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);
  new FXColorWell(matrix32,FXRGB(0,0,0),owner,CALCWindow::ID_TEXTCOLOR_CLEARALL,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_COLUMN|LAYOUT_FILL_ROW,0,0,40,24);

  //Easter egg tab
  FXTabItem* tab4=new FXTabItem(tabbook,"&Answer",NULL);
  FXVerticalFrame* eastereggframe=new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  new FXLabel(eastereggframe,"to the Ultimate Question of Life, the Universe and Everything");
  new FXHorizontalSeparator(eastereggframe,SEPARATOR_LINE|LAYOUT_FILL_X);

  FXVerticalFrame *frame3=new FXVerticalFrame(eastereggframe,PACK_UNIFORM_WIDTH|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  new FXCheckButton(frame3,"Hitchicker's Guide\tDon't Panic",owner,CALCWindow::ID_EASTEREGG1,ICON_BEFORE_TEXT|JUSTIFY_LEFT);
  new FXCheckButton(frame3,"I Ching Calculator\tA Suffusion of Yellow",owner,CALCWindow::ID_EASTEREGG2,ICON_BEFORE_TEXT|JUSTIFY_LEFT);

  tab4->hide();
  tab4->setTarget(this);
  tab4->setSelector(ID_EASTEREGGS);
}

CALCPrefsBox::~CALCPrefsBox()
{
}

void CALCPrefsBox::setPrefTab(FXuint tab)
{
  tabbook->handle(this,MKUINT(FXTabBar::ID_OPEN_FIRST+tab,SEL_COMMAND),NULL);
}

long CALCPrefsBox::onCmdDefaults(FXObject*,FXSelector,void*)
{
  switch(tabbook->getCurrent())
  {
    case 0:
      getOwner()->handle(this,MKUINT(CALCWindow::ID_DEFAULTDISPLAYFONT,SEL_COMMAND),NULL);
      getOwner()->handle(this,MKUINT(CALCWindow::ID_DEFAULTBUTTONFONT,SEL_COMMAND),NULL);
      getOwner()->handle(this,MKUINT(CALCWindow::ID_DEFAULTMODE,SEL_COMMAND),NULL);
      getOwner()->handle(this,MKUINT(CALCWindow::ID_DEFAULTWIDTH,SEL_COMMAND),NULL);
      getOwner()->handle(this,MKUINT(CALCWindow::ID_DEFAULTHEIGHT,SEL_COMMAND),NULL);
      getOwner()->handle(this,MKUINT(CALCWindow::ID_DEFAULTNUMDIGITS,SEL_COMMAND),NULL);
      break;
    case 1:
      getOwner()->handle(this,MKUINT(CALCWindow::ID_DEFAULTCOLORS,SEL_COMMAND),NULL);
      break;
    case 2:
      getOwner()->handle(this,MKUINT(CALCWindow::ID_DEFAULTTEXTCOLORS,SEL_COMMAND),NULL);
      break;
    case 3:
      getOwner()->handle(this,MKUINT(CALCWindow::ID_DEFAULTEASTEREGGS,SEL_COMMAND),NULL);
      break;
  }
  return 1;
}

long CALCPrefsBox::onUpdEasterEggs(FXObject* sender,FXSelector,void*)
{
  CALCdouble value=0.0;
  getOwner()->handle(this,MKUINT(CALCWindow::ID_GETLCDVALUE,SEL_COMMAND),(void*)&value);

  FXuint msg=(value==(CALCdouble)42.0) ? ID_SHOW : ID_HIDE;
  sender->handle(this,MKUINT(msg,SEL_COMMAND),NULL);
  return 1;
}
